import java.util.*;

import javax.persistence.*;

public final class OutOfMemoryWhenPersistAndFlushBigDataInSingleTransaction {

	public static void main(String[] args) {
		System.out.println("start");
		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("objectdb:$objectdb/db/test.tmp;drop");
		EntityManager em = emf.createEntityManager();

		// Persist an entity with an embeddable:
		em.getTransaction().begin();
		for (int i = 0; i < 5000000; i++) {
			
            Message msg = new MessageImpl();
            msg.setTypeID(UUID.randomUUID().toString()
                    + "1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890");
			em.persist(msg);
			
			if (i % 10000 == 0) {
				em.flush();
				em.clear();
				System.out.println("flush: " + i);
			}
		}
		em.getTransaction().commit();

		em.close();
		emf.close();
	}

	@Entity
	public static class MyEntity {
		
		@Basic
		private String value;
		
		@Basic
		private String value1;
		
		public void setValue(String value) {
			this.value = value;
		}
		
		public void setValue1(String value1) {
			this.value1 = value1;
		}
	}
	
	@Entity
	public static class MessageImpl extends ModelElementImpl implements Message {

	    @ManyToOne (cascade = { CascadeType.REFRESH },
	        targetEntity = ModelElementImpl.class)
	    private ModelElement causingObject;

	    @Basic
	    private MessageSeverity severity;

	    @Basic
	    private String typeID;

	    @ElementCollection (targetClass = TokenImpl.class)
	    private List<Token> messageTokens;

	    @ElementCollection (targetClass = TokenImpl.class)
	    private List<Token> hintTokens;

	    @Basic
	    @Temporal (TemporalType.TIMESTAMP)
	    private Date date;

	    /**
	     * Construct a new message and set its date to this time of instantiation.
	     */
	    public MessageImpl() {
	        super();
	        this.date = new Date();
	    }

	    @Override
	    public MessageSeverity getSeverity() {
	        return severity;
	    }

	    @Override
	    public void setSeverity(MessageSeverity severity) {
	        this.severity = severity;
	    }

	    @Override
	    public void setTypeID(String typeID) {
	        this.typeID = typeID;
	    }

	    @Override
	    public void setCausingObject(ModelElement causingObject) {
	        if (causingObject != null) {
	            this.causingObject = causingObject;
	        }
	    }

	    @Override
	    public String getTypeID() {
	        return this.typeID;
	    }

	    @Override
	    public ModelElement getCausingObject() {
	        return causingObject;
	    }

	    @Override
	    public void setMessage(List<Token> messageTokens) {
	        this.messageTokens = messageTokens;
	    }

	    @Override
	    public void setHint(List<Token> hintTokens) {
	        this.hintTokens = hintTokens;
	    }

	    @Override
	    public List<Token> getMessage() {
	        return messageTokens;
	    }

	    @Override
	    public List<Token> getHint() {
	        return hintTokens;
	    }

	    @Override
	    public void addMessageToken(Token t) {
	        messageTokens.add(t);
	    }

	    @Override
	    public void addHintToken(Token t) {
	        hintTokens.add(t);
	    }

	    @Override
	    public String toString() {
	        StringBuffer buf = new StringBuffer();
	        buf.append("[message: type: ");
	        buf.append(this.getTypeID());
	        buf.append(", ");
	        for (Message.Token t : getMessage()) {
	            buf.append(t.toString());
	        }
	        buf.append(", hint: ");
	        if (getHint() != null) {
	            for (Message.Token t : getHint()) {
	                buf.append(t.toString());
	            }
	        }
	        buf.append("]");

	        return buf.toString();
	    }

	    @Override
	    public Date getDate() {
	        return this.date;
	    }

	    /**
	     * Sets the date of creation of this message.
	     *
	     * @param created date that this message was created.
	     */
	    public void setDate(Date created) {
	        this.date = created;
	    }
	}

	@Entity
	public static abstract class ModelElementImpl implements ModelElement {

	    @Id
	    private String uid;

	    /**
	     * JPA Entity Default constructor
	     * <p>The default constructor must be a public or protected zero-argument constructor.</p>
	     */
	    protected ModelElementImpl() {
	        this.uid = UUID.randomUUID().toString().toUpperCase().replaceAll("-", "");
	    }

	    @Override
	    public int hashCode() {
	        if (uid == null) {
	            return super.hashCode();
	        }

	        return uid.hashCode();
	    }

	    @Override
	    public boolean equals(Object obj) {
	        if (obj instanceof ModelElement && this.getUid() != null) {
	            return this.getUid().equals(((ModelElement)obj).getUid());
	        }

	        return super.equals(obj);
	    }

	    @Override
	    public String getURI() {
	        throw new UnsupportedOperationException();
	    }

	    @Override
	    public final String getUid() {
	        return this.uid;
	    }
	}

	/**
	 * Instances of implementations of this class represent messages that have been issued via the MessageService, have been
	 * stored, and are ready to be shown via the UI.
	 */
	public static interface Message extends ModelElement {

	    /**
	     * The time stamp of the time, when the message was created.
	     * 
	     * @return time stamp of message creation.
	     */
	    public Date getDate();

	    /**
	     * The severity of this message.
	     * 
	     * @return message severity.
	     */
	    public MessageSeverity getSeverity();

	    /**
	     * Messages are represented by a sequence of tokens. All tokens have a string representation that must be suitable
	     * for the UI display.
	     */
	    public static interface Token {

	        /**
	         * Serialization.
	         * 
	         * @return the string for this part of the message.
	         */
	        public String toString();
	    }

	    /**
	     * LinkTokens additionally may have an object that they refer to.
	     */
	    public static interface LinkToken extends Token {

	        /**
	         * Since messages may refer to objects that are no longer accessible, this link may be become broken, i.e. this
	         * method may return null.
	         * 
	         * @return the object pointed to by this link.
	         */
	        public ModelElement getLinkedObject();
	    }

	    /**
	     * Get the ID of the message type of this message.
	     * 
	     * @return the ID of the message type of this message.
	     */
	    public String getTypeID();

	    /**
	     * To get a string representation of this message, concatenate the toString() outputs of the list of tokens returned
	     * by getMessage() -- without intermediate spaces! To add hyperlinks, add special handling for any LinkTokens
	     * occurring in this list.
	     * 
	     * @return the list of tokens that together constitute this message.
	     */
	    public List<Token> getMessage();

	    /**
	     * Store a list of tokens as the contents of the message, whose concatenation represents the message string.
	     * 
	     * @param messageTokens that represent the message text (and links).
	     */
	    public void setMessage(List<Token> messageTokens);

	    /**
	     * To get a string representation of this message's hint text, concatenate the toString() outputs of the list of
	     * tokens returned by getHint(). To add hyperlinks, add special handling for any LinkTokens occurring in this list.
	     * 
	     * @return the list of tokens that together constitute the hint of this message.
	     */
	    public List<Token> getHint();

	    /**
	     * Store a list of tokens as the contents of the hint, whose concatenation represents the message string.
	     * 
	     * @param hintTokens that represent the hint text (and links).
	     */
	    public void setHint(List<Token> hintTokens);

	    /**
	     * Get the message causing object.
	     * 
	     * @return the object that has caused this message, null if this object is no longer accessible.
	     */
	    public ModelElement getCausingObject();

	    /**
	     * Set severity of this message.
	     * 
	     * @param severity message severity.
	     */
	    public void setSeverity(MessageSeverity severity);

	    /**
	     * Set the ID of the message type of this message.
	     * 
	     * @param typeID the ID of the message type of this message.
	     */
	    public void setTypeID(String typeID);

	    /**
	     * Set the message causing object.
	     * 
	     * @param causingObject the object that has caused this message, null if this object is no longer accessible.
	     */
	    public void setCausingObject(ModelElement causingObject);

	    /**
	     * Add a token to the message.
	     * 
	     * @param t the token that is to be added to the message.
	     */
	    public void addMessageToken(Message.Token t);

	    /**
	     * Add a token to the hint.
	     * 
	     * @param t the token that is to be added to the hint.
	     */
	    public void addHintToken(Message.Token t);

	}

	public static abstract interface ModelElement {

	    /**
	     * Get the Uniform Resource Identifier for this object.
	     * 
	     * @return Uniform Resource Identifier
	     */
	    public String getURI();

	    /**
	     * Get the Universally Unique Identifier for this object.
	     * 
	     * @return Universally Unique Identifier
	     */
	    public String getUid();

	    /**
	     * <p>Unless specialized further by subclasses, any two ModelElement objects are equal if they have the same UID.
	     * Unless specialized further, this method is therefore guaranteed to return true whenever the given {@code obj} has
	     * the same UID as {@code this}. Subclasses may decide that ModelElement objects are equal even if they have
	     * different UIDs.</p>
	     * 
	     * <h3>Generic inherited javadoc follows.</h3> {@inheritDoc}
	     */
	    @Override
	    public boolean equals(Object obj);

	    /**
	     * <p>For any two ModelElements with the same non-null UID, this method is guaranteed to return the same hash
	     * code.</p>
	     * 
	     * <h3>Generic inherited javadoc follows.</h3> {@inheritDoc}
	     */
	    @Override
	    public int hashCode();
	}

	public static enum MessageSeverity {
	    INFO, WARNING, ERROR
	}

	@Embeddable
	public static class TokenImpl implements Token {

	    @Basic
	    String string;

	    /**
	     * Create an object
	     */
	    protected TokenImpl() {
	    }

	    /**
	     * Initialize a Token from a string.
	     *
	     * @param string representation of this token.
	     */
	    public TokenImpl(String string) {
	        this.string = string;
	    }

	    @Override
	    public String toString() {
	        return string;
	    }
	}

    public static interface Token {

        /**
         * Serialization.
         * 
         * @return the string for this part of the message.
         */
        public String toString();
    }
}
